# -*- coding: utf-8 -*-
"""pytypo - Spell correction

This module corrects spelling mistake.
That feature is based on TYPO CORPUS (http://luululu.com/tweet/)
and Wikipedia (https://en.wikipedia.org/wiki/Wikipedia:Lists_of_common_misspellings/For_machines)

Also this module normalizes lengthened English expression having repeating letters.
(e.g., this module converts "cooooooooooooooollllllllllllll" to "cool")

That feature is based on the following paper:
Samuel Brody and Nicholas Diakopoulos.
Cooooooooooooooollllllllllllll!!!!!!!!!!!!!! using word lengthening to detect sentiment in microblogs.
In EMNLP2011, pp. 562-570, 2011.
http://aclweb.org/anthology//D/D11/D11-1052.pdf


Author:
    Yukino Ikegami

Lisence:
    MIT License

Usage:
    import pytypo
    text = pytypo.correct(text) # Normalization for a word
    text = pytypo.normalize_sentence(text) # Normalization for a sentence
"""
from . import pytypo

VERSION = (0, 3, 0)
__version__ = '0.3.0'
__all__ = ['correct', 'correct_sentence', 'cut_repeat']

correct = pytypo.correct
correct_sentence = pytypo.correct_sentence
cut_repeat = pytypo.cut_repeat



# -*- coding: utf-8 -*-
from sys import version_info
import re
from . import _conv_table

re_symbol = re.compile("[^a-zA-Z0-9']{2,256}$")

if version_info < (3, 0, 0):
    range = xrange


def cut_repeat(text, threshold):
    """Reduce repeated characters until threshold
    Param:
        <str> text
        <int> threshold
    Return:
        <str> result
    """
    text = list(text)
    result = text[0]
    count = 0
    for i in range(1, len(text)):
        if text[i - 1] == text[i]:
            count += 1
            if count < threshold:
                result += text[i]
        else:
            count = 0
            result += text[i]
    return result


def correct(word):
    """Normalize repeat expression for English
    Param:
        <str> word
    Return:
        <str> normalized_word
    """
    suffix_symbol = re_symbol.findall(word)
    if suffix_symbol:
        word = word.replace(suffix_symbol[0], '')
    word = cut_repeat(word, 2)
    normalized_word = _conv_table.eng_lengthened.get(word, word)
    if normalized_word == word:
        normalized_word = _conv_table.eng_typo.get(word, word)
    if suffix_symbol:
        return (normalized_word + cut_repeat(suffix_symbol[0], 1))
    return normalized_word


def correct_sentence(sentence):
    """Normalize for each word
    Param:
        <str> sentence
    Return:
        <str> normalized_sentence
    """
    return ' '.join([correct(word) for word in sentence.split()])
